@@ -55,6 +55,8 @@
 #endif
 #endif
 
+#include <private/qcore_unix_p.h>
+
 #include <QtCore/qelapsedtimer.h>
 #include <QtCore/qsocketnotifier.h>
 #include <QtCore/qmap.h>
@@ -133,7 +135,7 @@ protected:
     bool event(QEvent *e) Q_DECL_OVERRIDE {
         bool ret = QSocketNotifier::event(e);
         if (ret)
-            dptr->writeNotification();
+            dptr->completeAsyncWrite();
         return ret;
     }
 
@@ -176,6 +178,8 @@ QSerialPortPrivate::QSerialPortPrivate(QSerialPort *q)
     , readPortNotifierStateSet(false)
     , emittedReadyRead(false)
     , emittedBytesWritten(false)
+    , pendingBytesWritten(0)
+    , writeSequenceStarted(false)
 {
 }
 
@@ -213,7 +217,7 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
         break;
     }
 
-    descriptor = ::open(systemLocation.toLocal8Bit().constData(), flags);
+    descriptor = qt_safe_open(systemLocation.toLocal8Bit().constData(), flags);
 
     if (descriptor == -1) {
         q->setError(decodeSystemError());
@@ -227,7 +231,8 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
     }
 
 #ifdef TIOCEXCL
-    ::ioctl(descriptor, TIOCEXCL);
+    if (::ioctl(descriptor, TIOCEXCL) == -1)
+        q->setError(decodeSystemError());
 #endif
 
     if (::tcgetattr(descriptor, &restoredTermios) == -1) {
@@ -236,7 +241,15 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
     }
 
     currentTermios = restoredTermios;
+#ifdef Q_OS_SOLARIS
+    currentTermios.c_iflag &= ~(IMAXBEL|IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
+    currentTermios.c_oflag &= ~OPOST;
+    currentTermios.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
+    currentTermios.c_cflag &= ~(CSIZE|PARENB);
+    currentTermios.c_cflag |= CS8;
+#else
     ::cfmakeraw(&currentTermios);
+#endif
     currentTermios.c_cflag |= CLOCAL;
     currentTermios.c_cc[VTIME] = 0;
     currentTermios.c_cc[VMIN] = 0;
@@ -247,27 +260,38 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
     if (!updateTermios())
         return false;
 
+#ifdef Q_OS_LINUX
+    isCustomBaudRateSupported = (::ioctl(descriptor, TIOCGSERIAL, &currentSerialInfo) != -1)
+            && (::ioctl(descriptor, TIOCSSERIAL, &currentSerialInfo) != -1);
+#endif
+
     setExceptionNotificationEnabled(true);
 
     if ((flags & O_WRONLY) == 0)
         setReadNotificationEnabled(true);
 
-    detectDefaultSettings();
     return true;
 }
 
 void QSerialPortPrivate::close()
 {
+    Q_Q(QSerialPort);
+
     if (settingsRestoredOnClose) {
-        ::tcsetattr(descriptor, TCSANOW, &restoredTermios);
+        if (::tcsetattr(descriptor, TCSANOW, &restoredTermios) == -1)
+            q->setError(decodeSystemError());
+
 #ifdef Q_OS_LINUX
-        if (isCustomBaudRateSupported)
-            ::ioctl(descriptor, TIOCSSERIAL, &restoredSerialInfo);
+        if (isCustomBaudRateSupported) {
+            if (::ioctl(descriptor, TIOCSSERIAL, &restoredSerialInfo) == -1)
+                q->setError(decodeSystemError());
+        }
 #endif
     }
 
 #ifdef TIOCNXCL
-    ::ioctl(descriptor, TIOCNXCL);
+    if (::ioctl(descriptor, TIOCNXCL) == -1)
+        q->setError(decodeSystemError());
 #endif
 
     if (readNotifier) {
@@ -288,13 +312,16 @@ void QSerialPortPrivate::close()
         exceptionNotifier = 0;
     }
 
-    ::close(descriptor);
+    if (qt_safe_close(descriptor) == -1)
+        q->setError(decodeSystemError());
 
     if (lockFileScopedPointer->isLocked())
         lockFileScopedPointer->unlock();
 
     descriptor = -1;
     isCustomBaudRateSupported = false;
+    pendingBytesWritten = 0;
+    writeSequenceStarted = false;
 }
 
 QSerialPort::PinoutSignals QSerialPortPrivate::pinoutSignals()
@@ -358,19 +385,33 @@ QSerialPort::PinoutSignals QSerialPortPrivate::pinoutSignals()
 
 bool QSerialPortPrivate::setDataTerminalReady(bool set)
 {
+    Q_Q(QSerialPort);
+
     int status = TIOCM_DTR;
-    return ::ioctl(descriptor, set ? TIOCMBIS : TIOCMBIC, &status) != -1;
+    if (::ioctl(descriptor, set ? TIOCMBIS : TIOCMBIC, &status) == -1) {
+        q->setError(decodeSystemError());
+        return false;
+    }
+
+    return true;
 }
 
 bool QSerialPortPrivate::setRequestToSend(bool set)
 {
+    Q_Q(QSerialPort);
+
     int status = TIOCM_RTS;
-    return ::ioctl(descriptor, set ? TIOCMBIS : TIOCMBIC, &status) != -1;
+    if (::ioctl(descriptor, set ? TIOCMBIS : TIOCMBIC, &status) == -1) {
+        q->setError(decodeSystemError());
+        return false;
+    }
+
+    return true;
 }
 
 bool QSerialPortPrivate::flush()
 {
-    return writeNotification()
+    return completeAsyncWrite()
 #ifndef Q_OS_ANDROID
             && (::tcdrain(descriptor) != -1);
 #else
@@ -380,54 +421,45 @@ bool QSerialPortPrivate::flush()
 
 bool QSerialPortPrivate::clear(QSerialPort::Directions directions)
 {
-    return ::tcflush(descriptor, (directions == QSerialPort::AllDirections)
-                     ? TCIOFLUSH : (directions & QSerialPort::Input) ? TCIFLUSH : TCOFLUSH) != -1;
+    Q_Q(QSerialPort);
+
+    if (::tcflush(descriptor, (directions == QSerialPort::AllDirections)
+                     ? TCIOFLUSH : (directions & QSerialPort::Input) ? TCIFLUSH : TCOFLUSH) == -1) {
+        q->setError(decodeSystemError());
+        return false;
+    }
+
+    return true;
 }
 
 bool QSerialPortPrivate::sendBreak(int duration)
 {
-    return ::tcsendbreak(descriptor, duration) != -1;
+    Q_Q(QSerialPort);
+
+    if (::tcsendbreak(descriptor, duration) == -1) {
+        q->setError(decodeSystemError());
+        return false;
+    }
+
+    return true;
 }
 
 bool QSerialPortPrivate::setBreakEnabled(bool set)
 {
-    return ::ioctl(descriptor, set ? TIOCSBRK : TIOCCBRK) != -1;
+    Q_Q(QSerialPort);
+
+    if (::ioctl(descriptor, set ? TIOCSBRK : TIOCCBRK) == -1) {
+        q->setError(decodeSystemError());
+        return false;
+    }
+
+    return true;
 }
 
-qint64 QSerialPortPrivate::systemInputQueueSize () const
+void QSerialPortPrivate::startWriting()
 {
-    int nbytes = 0;
-#ifdef TIOCINQ
-    if (::ioctl(descriptor, TIOCINQ, &nbytes) == -1)
-        return -1;
-#endif
-    return nbytes;
-}
-
-qint64 QSerialPortPrivate::systemOutputQueueSize () const
-{
-    int nbytes = 0;
-#ifdef TIOCOUTQ
-    if (::ioctl(descriptor, TIOCOUTQ, &nbytes) == -1)
-        return -1;
-#endif
-    return nbytes;
-}
-
-qint64 QSerialPortPrivate::writeToBuffer(const char *data, qint64 maxSize)
-{
-    char *ptr = writeBuffer.reserve(maxSize);
-    if (maxSize == 1)
-        *ptr = *data;
-    else
-        ::memcpy(ptr, data, maxSize);
-
-    const qint64 written = maxSize;
-
-    if (!writeBuffer.isEmpty() && !isWriteNotificationEnabled())
+    if (!isWriteNotificationEnabled())
         setWriteNotificationEnabled(true);
-
-    return written;
 }
 
 bool QSerialPortPrivate::waitForReadyRead(int msecs)
@@ -455,7 +487,7 @@ bool QSerialPortPrivate::waitForReadyRead(int msecs)
         }
 
         if (readyToWrite)
-            writeNotification();
+            completeAsyncWrite();
 
     } while (msecs == -1 || timeoutValue(msecs, stopWatch.elapsed()) > 0);
     return false;
@@ -465,7 +497,7 @@ bool QSerialPortPrivate::waitForBytesWritten(int msecs)
 {
     Q_Q(QSerialPort);
 
-    if (writeBuffer.isEmpty())
+    if (writeBuffer.isEmpty() && pendingBytesWritten <= 0)
         return false;
 
     QElapsedTimer stopWatch;
@@ -487,11 +519,20 @@ bool QSerialPortPrivate::waitForBytesWritten(int msecs)
             return false;
 
         if (readyToWrite)
-            return writeNotification();
+            return completeAsyncWrite();
     }
     return false;
 }
 
+bool QSerialPortPrivate::setBaudRate()
+{
+    if (inputBaudRate == outputBaudRate)
+        return setBaudRate(inputBaudRate, QSerialPort::AllDirections);
+
+    return (setBaudRate(inputBaudRate, QSerialPort::Input)
+        && setBaudRate(outputBaudRate, QSerialPort::Output));
+}
+
 bool QSerialPortPrivate::setBaudRate(qint32 baudRate, QSerialPort::Directions directions)
 {
     Q_Q(QSerialPort);
@@ -640,9 +681,9 @@ bool QSerialPortPrivate::setStopBits(QSerialPort::StopBits stopBits)
     return updateTermios();
 }
 
-bool QSerialPortPrivate::setFlowControl(QSerialPort::FlowControl flow)
+bool QSerialPortPrivate::setFlowControl(QSerialPort::FlowControl flowControl)
 {
-    switch (flow) {
+    switch (flowControl) {
     case QSerialPort::NoFlowControl:
         currentTermios.c_cflag &= ~CRTSCTS;
         currentTermios.c_iflag &= ~(IXON | IXOFF | IXANY);
@@ -766,23 +807,15 @@ bool QSerialPortPrivate::readNotification()
     return true;
 }
 
-bool QSerialPortPrivate::writeNotification()
+bool QSerialPortPrivate::startAsyncWrite()
 {
     Q_Q(QSerialPort);
 
-    const int tmp = writeBuffer.size();
-
-    if (writeBuffer.isEmpty()) {
-        setWriteNotificationEnabled(false);
-        return false;
-    }
-
-    int nextSize = writeBuffer.nextDataBlockSize();
-
-    const char *ptr = writeBuffer.readPointer();
+    if (writeBuffer.isEmpty() || writeSequenceStarted)
+        return true;
 
     // Attempt to write it all in one chunk.
-    qint64 written = writeToPort(ptr, nextSize);
+    qint64 written = writeToPort(writeBuffer.readPointer(), writeBuffer.nextDataBlockSize());
     if (written < 0) {
         QSerialPort::SerialPortError error = decodeSystemError();
         if (error != QSerialPort::ResourceError)
@@ -791,21 +824,36 @@ bool QSerialPortPrivate::writeNotification()
         return false;
     }
 
-    // Remove what we wrote so far.
     writeBuffer.free(written);
-    if (written > 0) {
-        // Don't emit bytesWritten() recursively.
+    pendingBytesWritten += written;
+    writeSequenceStarted = true;
+
+    if (!isWriteNotificationEnabled())
+        setWriteNotificationEnabled(true);
+    return true;
+}
+
+bool QSerialPortPrivate::completeAsyncWrite()
+{
+    Q_Q(QSerialPort);
+
+    if (pendingBytesWritten > 0) {
         if (!emittedBytesWritten) {
             emittedBytesWritten = true;
-            emit q->bytesWritten(written);
+            emit q->bytesWritten(pendingBytesWritten);
+            pendingBytesWritten = 0;
             emittedBytesWritten = false;
         }
     }
 
-    if (writeBuffer.isEmpty())
-        setWriteNotificationEnabled(false);
+    writeSequenceStarted = false;
 
-    return (writeBuffer.size() < tmp);
+    if (writeBuffer.isEmpty()) {
+        setWriteNotificationEnabled(false);
+        return true;
+    }
+
+    return startAsyncWrite();
 }
 
 void QSerialPortPrivate::exceptionNotification()
@@ -827,99 +875,6 @@ bool QSerialPortPrivate::updateTermios()
     return true;
 }
 
-void QSerialPortPrivate::detectDefaultSettings()
-{
-    // Detect baud rate.
-    const speed_t inputUnixBaudRate = ::cfgetispeed(&currentTermios);
-    const speed_t outputUnixBaudRate = ::cfgetospeed(&currentTermios);
-    bool isCustomBaudRateCurrentSet = false;
-
-#ifdef Q_OS_LINUX
-    // try detect the ability to support custom baud rate
-    isCustomBaudRateSupported = ::ioctl(descriptor, TIOCGSERIAL, &currentSerialInfo) != -1
-            && ::ioctl(descriptor, TIOCSSERIAL, &currentSerialInfo) != -1;
-
-    if (isCustomBaudRateSupported) {
-        restoredSerialInfo = currentSerialInfo;
-
-        // assume that the baud rate is a custom
-        isCustomBaudRateCurrentSet = inputUnixBaudRate == B38400 && outputUnixBaudRate == B38400;
-
-        if (isCustomBaudRateCurrentSet) {
-            if ((currentSerialInfo.flags & ASYNC_SPD_CUST)
-                    && currentSerialInfo.custom_divisor > 0) {
-
-                // yes, speed is really custom
-                inputBaudRate = currentSerialInfo.baud_base / currentSerialInfo.custom_divisor;
-                outputBaudRate = inputBaudRate;
-            } else {
-                // no, we were wrong and the speed is a standard 38400 baud
-                isCustomBaudRateCurrentSet = false;
-            }
-        }
-    }
-#else
-    // other *nix
-#endif
-    if (!isCustomBaudRateSupported || !isCustomBaudRateCurrentSet) {
-        inputBaudRate = QSerialPortPrivate::baudRateFromSetting(inputUnixBaudRate);
-        outputBaudRate = QSerialPortPrivate::baudRateFromSetting(outputUnixBaudRate);
-    }
-
-    // Detect databits.
-    switch (currentTermios.c_cflag & CSIZE) {
-    case CS5:
-        dataBits = QSerialPort::Data5;
-        break;
-    case CS6:
-        dataBits = QSerialPort::Data6;
-        break;
-    case CS7:
-        dataBits = QSerialPort::Data7;
-        break;
-    case CS8:
-        dataBits = QSerialPort::Data8;
-        break;
-    default:
-        qWarning("%s: Unexpected data bits settings", Q_FUNC_INFO);
-        dataBits = QSerialPort::Data8;
-        break;
-    }
-
-    // Detect parity.
-#ifdef CMSPAR
-    if (currentTermios.c_cflag & CMSPAR) {
-        parity = currentTermios.c_cflag & PARODD ?
-                    QSerialPort::MarkParity : QSerialPort::SpaceParity;
-    } else {
-#endif
-        if (currentTermios.c_cflag & PARENB) {
-            parity = currentTermios.c_cflag & PARODD ?
-                        QSerialPort::OddParity : QSerialPort::EvenParity;
-        } else {
-            parity = QSerialPort::NoParity;
-        }
-#ifdef CMSPAR
-    }
-#endif
-
-    // Detect stopbits.
-    stopBits = currentTermios.c_cflag & CSTOPB ?
-                QSerialPort::TwoStop : QSerialPort::OneStop;
-
-    // Detect flow control.
-    if ((!(currentTermios.c_cflag & CRTSCTS)) && (!(currentTermios.c_iflag & (IXON | IXOFF | IXANY))))
-        flow = QSerialPort::NoFlowControl;
-    else if ((!(currentTermios.c_cflag & CRTSCTS)) && (currentTermios.c_iflag & (IXON | IXOFF | IXANY)))
-        flow = QSerialPort::SoftwareControl;
-    else if ((currentTermios.c_cflag & CRTSCTS) && (!(currentTermios.c_iflag & (IXON | IXOFF | IXANY))))
-        flow = QSerialPort::HardwareControl;
-    else {
-        qWarning("%s: Unexpected flow control settings", Q_FUNC_INFO);
-        flow = QSerialPort::NoFlowControl;
-    }
-}
-
 QSerialPort::SerialPortError QSerialPortPrivate::decodeSystemError() const
 {
     QSerialPort::SerialPortError error;
@@ -1054,7 +1009,7 @@ qint64 QSerialPortPrivate::readFromPort(char *data, qint64 maxSize)
     if (parity != QSerialPort::MarkParity
             && parity != QSerialPort::SpaceParity) {
 #endif
-        bytesRead = ::read(descriptor, data, maxSize);
+        bytesRead = qt_safe_read(descriptor, data, maxSize);
     } else {// Perform parity emulation.
         bytesRead = readPerChar(data, maxSize);
     }
@@ -1066,11 +1021,11 @@ qint64 QSerialPortPrivate::writeToPort(const char *data, qint64 maxSize)
 {
     qint64 bytesWritten = 0;
 #if defined (CMSPAR)
-    bytesWritten = ::write(descriptor, data, maxSize);
+    bytesWritten = qt_safe_write(descriptor, data, maxSize);
 #else
     if (parity != QSerialPort::MarkParity
             && parity != QSerialPort::SpaceParity) {
-        bytesWritten = ::write(descriptor, data, maxSize);
+        bytesWritten = qt_safe_write(descriptor, data, maxSize);
     } else {// Perform parity emulation.
         bytesWritten = writePerChar(data, maxSize);
     }
@@ -1107,7 +1062,7 @@ qint64 QSerialPortPrivate::writePerChar(const char *data, qint64 maxSize)
                 break;
         }
 
-        int r = ::write(descriptor, data, 1);
+        int r = qt_safe_write(descriptor, data, 1);
         if (r < 0)
             return -1;
         if (r > 0) {
@@ -1133,7 +1088,7 @@ qint64 QSerialPortPrivate::readPerChar(char *data, qint64 maxSize)
     int prefix = 0;
     while (ret < maxSize) {
 
-        qint64 r = ::read(descriptor, data, 1);
+        qint64 r = qt_safe_read(descriptor, data, 1);
         if (r < 0) {
             if (errno == EAGAIN) // It is ok for nonblocking mode.
                 break;
