@@ -98,11 +98,11 @@ static void initializeOverlappedStructure(OVERLAPPED &overlapped)
 
 QSerialPortPrivate::QSerialPortPrivate(QSerialPort *q)
     : QSerialPortPrivateData(q)
-    , descriptor(INVALID_HANDLE_VALUE)
+    , handle(INVALID_HANDLE_VALUE)
     , parityErrorOccurred(false)
     , readChunkBuffer(ReadChunkSize, 0)
     , readyReadEmitted(0)
-    , writeSequenceStarted(false)
+    , writeStarted(false)
     , communicationNotifier(new QWinEventNotifier(q))
     , readCompletionNotifier(new QWinEventNotifier(q))
     , writeCompletionNotifier(new QWinEventNotifier(q))
@@ -115,7 +115,7 @@ QSerialPortPrivate::QSerialPortPrivate(QSerialPort *q)
         q->setError(decodeSystemError());
     else {
         communicationNotifier->setHandle(communicationOverlapped.hEvent);
-        q->connect(communicationNotifier, SIGNAL(activated(HANDLE)), q, SLOT(_q_canCompleteCommunication()));
+        q->connect(communicationNotifier, SIGNAL(activated(HANDLE)), q, SLOT(_q_completeAsyncCommunication()));
     }
 
     ::ZeroMemory(&readCompletionOverlapped, sizeof(readCompletionOverlapped));
@@ -124,7 +124,7 @@ QSerialPortPrivate::QSerialPortPrivate(QSerialPort *q)
         q->setError(decodeSystemError());
     else {
         readCompletionNotifier->setHandle(readCompletionOverlapped.hEvent);
-        q->connect(readCompletionNotifier, SIGNAL(activated(HANDLE)), q, SLOT(_q_canCompleteRead()));
+        q->connect(readCompletionNotifier, SIGNAL(activated(HANDLE)), q, SLOT(_q_completeAsyncRead()));
     }
 
     ::ZeroMemory(&writeCompletionOverlapped, sizeof(writeCompletionOverlapped));
@@ -133,7 +133,7 @@ QSerialPortPrivate::QSerialPortPrivate(QSerialPort *q)
         q->setError(decodeSystemError());
     else {
         writeCompletionNotifier->setHandle(writeCompletionOverlapped.hEvent);
-        q->connect(writeCompletionNotifier, SIGNAL(activated(HANDLE)), q, SLOT(_q_canCompleteWrite()));
+        q->connect(writeCompletionNotifier, SIGNAL(activated(HANDLE)), q, SLOT(_q_completeAsyncWrite()));
     }
 }
 
@@ -151,15 +151,18 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
     if (mode & QIODevice::WriteOnly)
         desiredAccess |= GENERIC_WRITE;
 
-    descriptor = ::CreateFile(reinterpret_cast<const wchar_t*>(systemLocation.utf16()),
+    handle = ::CreateFile(reinterpret_cast<const wchar_t*>(systemLocation.utf16()),
                               desiredAccess, 0, NULL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
 
-    if (descriptor == INVALID_HANDLE_VALUE) {
+    if (handle == INVALID_HANDLE_VALUE) {
         q->setError(decodeSystemError());
         return false;
     }
 
-    if (!::GetCommState(descriptor, &restoredDcb)) {
+    ::ZeroMemory(&restoredDcb, sizeof(restoredDcb));
+    restoredDcb.DCBlength = sizeof(restoredDcb);
+
+    if (!::GetCommState(handle, &restoredDcb)) {
         q->setError(decodeSystemError());
         return false;
     }
@@ -172,10 +175,13 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
     currentDcb.fNull = FALSE;
     currentDcb.fErrorChar = FALSE;
 
+    if (currentDcb.fDtrControl ==  DTR_CONTROL_HANDSHAKE)
+        currentDcb.fDtrControl = DTR_CONTROL_DISABLE;
+
     if (!updateDcb())
         return false;
 
-    if (!::GetCommTimeouts(descriptor, &restoredCommTimeouts)) {
+    if (!::GetCommTimeouts(handle, &restoredCommTimeouts)) {
         q->setError(decodeSystemError());
         return false;
     }
@@ -192,23 +198,16 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
     if (mode & QIODevice::WriteOnly)
         writeCompletionNotifier->setEnabled(true);
 
-    if (!::SetCommMask(descriptor, originalEventMask)) {
+    if (!::SetCommMask(handle, originalEventMask)) {
         q->setError(decodeSystemError());
         return false;
     }
 
-    initializeOverlappedStructure(communicationOverlapped);
-    if (!::WaitCommEvent(descriptor, &triggeredEventMask, &communicationOverlapped)) {
-        const QSerialPort::SerialPortError error = decodeSystemError();
-        if (error != QSerialPort::NoError) {
-            q->setError(decodeSystemError());
-            return false;
-        }
-    }
+    if (!startAsyncCommunication())
+        return false;
 
     communicationNotifier->setEnabled(true);
 
-    detectDefaultSettings();
     return true;
 }
 
@@ -216,7 +215,7 @@ void QSerialPortPrivate::close()
 {
     Q_Q(QSerialPort);
 
-    if (!::CancelIo(descriptor))
+    if (!::CancelIo(handle))
         q->setError(decodeSystemError());
 
     readCompletionNotifier->setEnabled(false);
@@ -225,23 +224,23 @@ void QSerialPortPrivate::close()
 
     readBuffer.clear();
 
-    writeSequenceStarted = false;
+    writeStarted = false;
     writeBuffer.clear();
 
     readyReadEmitted = false;
     parityErrorOccurred = false;
 
     if (settingsRestoredOnClose) {
-        if (!::SetCommState(descriptor, &restoredDcb))
+        if (!::SetCommState(handle, &restoredDcb))
             q->setError(decodeSystemError());
-        else if (!::SetCommTimeouts(descriptor, &restoredCommTimeouts))
+        else if (!::SetCommTimeouts(handle, &restoredCommTimeouts))
             q->setError(decodeSystemError());
     }
 
-    if (!::CloseHandle(descriptor))
+    if (!::CloseHandle(handle))
         q->setError(decodeSystemError());
 
-    descriptor = INVALID_HANDLE_VALUE;
+    handle = INVALID_HANDLE_VALUE;
 }
 
 #endif // #ifndef Q_OS_WINCE
@@ -252,7 +251,7 @@ QSerialPort::PinoutSignals QSerialPortPrivate::pinoutSignals()
 
     DWORD modemStat = 0;
 
-    if (!::GetCommModemStatus(descriptor, &modemStat)) {
+    if (!::GetCommModemStatus(handle, &modemStat)) {
         q->setError(decodeSystemError());
         return QSerialPort::NoSignal;
     }
@@ -269,9 +268,10 @@ QSerialPort::PinoutSignals QSerialPortPrivate::pinoutSignals()
         ret |= QSerialPort::DataCarrierDetectSignal;
 
     DWORD bytesReturned = 0;
-    if (!::DeviceIoControl(descriptor, IOCTL_SERIAL_GET_DTRRTS, NULL, 0,
+    if (!::DeviceIoControl(handle, IOCTL_SERIAL_GET_DTRRTS, NULL, 0,
                           &modemStat, sizeof(modemStat),
                           &bytesReturned, NULL)) {
+        q->setError(decodeSystemError());
         return ret;
     }
 
@@ -285,93 +285,101 @@ QSerialPort::PinoutSignals QSerialPortPrivate::pinoutSignals()
 
 bool QSerialPortPrivate::setDataTerminalReady(bool set)
 {
-    return ::EscapeCommFunction(descriptor, set ? SETDTR : CLRDTR);
+    Q_Q(QSerialPort);
+
+    if (!::EscapeCommFunction(handle, set ? SETDTR : CLRDTR)) {
+        q->setError(decodeSystemError());
+        return false;
+    }
+
+    currentDcb.fDtrControl = set ? DTR_CONTROL_ENABLE : DTR_CONTROL_DISABLE;
+    return true;
 }
 
 bool QSerialPortPrivate::setRequestToSend(bool set)
 {
-    return ::EscapeCommFunction(descriptor, set ? SETRTS : CLRRTS);
+    Q_Q(QSerialPort);
+
+    if (!::EscapeCommFunction(handle, set ? SETRTS : CLRRTS)) {
+        q->setError(decodeSystemError());
+        return false;
+    }
+
+    return true;
 }
 
 #ifndef Q_OS_WINCE
 
 bool QSerialPortPrivate::flush()
 {
-    return startAsyncWrite() && ::FlushFileBuffers(descriptor);
+    Q_Q(QSerialPort);
+
+    bool returnValue = true;
+
+    if (!startAsyncWrite())
+        returnValue = false;
+
+    if (!::FlushFileBuffers(handle)) {
+        q->setError(decodeSystemError());
+        returnValue = false;
+    }
+
+    return returnValue;
+
 }
 
 bool QSerialPortPrivate::clear(QSerialPort::Directions directions)
 {
+    Q_Q(QSerialPort);
+
     DWORD flags = 0;
     if (directions & QSerialPort::Input)
         flags |= PURGE_RXABORT | PURGE_RXCLEAR;
     if (directions & QSerialPort::Output) {
         flags |= PURGE_TXABORT | PURGE_TXCLEAR;
-        writeSequenceStarted = false;
+        writeStarted = false;
     }
-    return ::PurgeComm(descriptor, flags);
+    if (!::PurgeComm(handle, flags)) {
+        q->setError(decodeSystemError());
+        return false;
+    }
+
+    return true;
 }
 
 #endif
 
 bool QSerialPortPrivate::sendBreak(int duration)
 {
-    // FIXME:
-    if (setBreakEnabled(true)) {
-        ::Sleep(duration);
-        if (setBreakEnabled(false))
-            return true;
-    }
-    return false;
+    if (!setBreakEnabled(true))
+        return false;
+
+    ::Sleep(duration);
+
+    if (!setBreakEnabled(false))
+        return false;
+
+    return true;
 }
 
 bool QSerialPortPrivate::setBreakEnabled(bool set)
-{
-    if (set)
-        return ::SetCommBreak(descriptor);
-    return ::ClearCommBreak(descriptor);
-}
-
-qint64 QSerialPortPrivate::systemInputQueueSize ()
 {
     Q_Q(QSerialPort);
 
-    COMSTAT cs;
-    ::ZeroMemory(&cs, sizeof(cs));
-    if (!::ClearCommError(descriptor, NULL, &cs)) {
+    if (set ? !::SetCommBreak(handle) : !::ClearCommBreak(handle)) {
         q->setError(decodeSystemError());
-        return -1;
+        return false;
     }
-    return cs.cbInQue;
-}
 
-qint64 QSerialPortPrivate::systemOutputQueueSize ()
-{
-    Q_Q(QSerialPort);
-
-    COMSTAT cs;
-    ::ZeroMemory(&cs, sizeof(cs));
-    if (!::ClearCommError(descriptor, NULL, &cs)) {
-        q->setError(decodeSystemError());
-        return -1;
-    }
-    return cs.cbOutQue;
+    return true;
 }
 
 #ifndef Q_OS_WINCE
 
-qint64 QSerialPortPrivate::writeToBuffer(const char *data, qint64 maxSize)
+void QSerialPortPrivate::startWriting()
 {
-    char *ptr = writeBuffer.reserve(maxSize);
-    if (maxSize == 1)
-        *ptr = *data;
-    else
-        ::memcpy(ptr, data, maxSize);
-
-    if (!writeSequenceStarted)
+    if (!writeStarted)
         startAsyncWrite();
-
-    return maxSize;
 }
 
 bool QSerialPortPrivate::waitForReadyRead(int msecs)
@@ -396,15 +404,15 @@ bool QSerialPortPrivate::waitForReadyRead(int msecs)
         }
 
         if (triggeredEvent == communicationOverlapped.hEvent) {
-            _q_canCompleteCommunication();
+            _q_completeAsyncCommunication();
         } else if (triggeredEvent == readCompletionOverlapped.hEvent) {
-            _q_canCompleteRead();
+            _q_completeAsyncRead();
             if (qint64(readBuffer.size()) != currentReadBufferSize)
                 currentReadBufferSize = readBuffer.size();
             else if (initialReadBufferSize != currentReadBufferSize)
                 return true;
         } else if (triggeredEvent == writeCompletionOverlapped.hEvent) {
-            _q_canCompleteWrite();
+            _q_completeAsyncWrite();
         } else {
             return false;
         }
@@ -435,11 +443,11 @@ bool QSerialPortPrivate::waitForBytesWritten(int msecs)
         }
 
         if (triggeredEvent == communicationOverlapped.hEvent) {
-             _q_canCompleteRead();
+             _q_completeAsyncRead();
         } else if (triggeredEvent == readCompletionOverlapped.hEvent) {
-            _q_canCompleteRead();
+            _q_completeAsyncRead();
         } else if (triggeredEvent == writeCompletionOverlapped.hEvent) {
-            _q_canCompleteWrite();
+            _q_completeAsyncWrite();
             return error == QSerialPort::NoError;
         } else {
             return false;
@@ -452,6 +460,11 @@ bool QSerialPortPrivate::waitForBytesWritten(int msecs)
 
 #endif // #ifndef Q_OS_WINCE
 
+bool QSerialPortPrivate::setBaudRate()
+{
+    return setBaudRate(inputBaudRate, QSerialPort::AllDirections);
+}
+
 bool QSerialPortPrivate::setBaudRate(qint32 baudRate, QSerialPort::Directions directions)
 {
     Q_Q(QSerialPort);
@@ -517,13 +530,13 @@ bool QSerialPortPrivate::setStopBits(QSerialPort::StopBits stopBits)
     return updateDcb();
 }
 
-bool QSerialPortPrivate::setFlowControl(QSerialPort::FlowControl flow)
+bool QSerialPortPrivate::setFlowControl(QSerialPort::FlowControl flowControl)
 {
     currentDcb.fInX = FALSE;
     currentDcb.fOutX = FALSE;
     currentDcb.fOutxCtsFlow = FALSE;
     currentDcb.fRtsControl = RTS_CONTROL_DISABLE;
-    switch (flow) {
+    switch (flowControl) {
     case QSerialPort::NoFlowControl:
         break;
     case QSerialPort::SoftwareControl:
@@ -548,13 +561,13 @@ bool QSerialPortPrivate::setDataErrorPolicy(QSerialPort::DataErrorPolicy policy)
 
 #ifndef Q_OS_WINCE
 
-void QSerialPortPrivate::_q_canCompleteCommunication()
+void QSerialPortPrivate::_q_completeAsyncCommunication()
 {
     Q_Q(QSerialPort);
 
     DWORD numberOfBytesTransferred = 0;
 
-    if (!::GetOverlappedResult(descriptor, &communicationOverlapped, &numberOfBytesTransferred, FALSE))
+    if (!::GetOverlappedResult(handle, &communicationOverlapped, &numberOfBytesTransferred, FALSE))
         q->setError(decodeSystemError());
 
     bool error = false;
@@ -580,41 +593,61 @@ void QSerialPortPrivate::_q_canCompleteCommunication()
         startAsyncRead();
 }
 
-void QSerialPortPrivate::_q_canCompleteRead()
+void QSerialPortPrivate::_q_completeAsyncRead()
 {
     Q_Q(QSerialPort);
 
     DWORD numberOfBytesTransferred = 0;
-    if (!::GetOverlappedResult(descriptor, &readCompletionOverlapped, &numberOfBytesTransferred, FALSE))
+    if (!::GetOverlappedResult(handle, &readCompletionOverlapped, &numberOfBytesTransferred, FALSE))
         q->setError(decodeSystemError());
 
-    completeAsyncRead(numberOfBytesTransferred);
+    if (numberOfBytesTransferred > 0) {
+
+        readBuffer.append(readChunkBuffer.left(numberOfBytesTransferred));
+
+        if (!emulateErrorPolicy())
+            emitReadyRead();
+    }
 
     // start async read for possible remainder into driver queue
-    if ((numberOfBytesTransferred > 0) && (policy == QSerialPort::IgnorePolicy)) {
+    if ((numberOfBytesTransferred > 0) && (policy == QSerialPort::IgnorePolicy))
         startAsyncRead();
-    } else { // driver queue is emplty, so startup wait comm event
-        initializeOverlappedStructure(communicationOverlapped);
-        if (!::WaitCommEvent(descriptor, &triggeredEventMask, &communicationOverlapped)) {
-            const QSerialPort::SerialPortError error = decodeSystemError();
-            if (error != QSerialPort::NoError) {
-                q->setError(decodeSystemError());
-            }
-        }
-    }
+    else // driver queue is emplty, so startup wait comm event
+        startAsyncCommunication();
 }
 
-void QSerialPortPrivate::_q_canCompleteWrite()
+void QSerialPortPrivate::_q_completeAsyncWrite()
 {
     Q_Q(QSerialPort);
 
     DWORD numberOfBytesTransferred = 0;
-    if (!::GetOverlappedResult(descriptor, &writeCompletionOverlapped, &numberOfBytesTransferred, FALSE)) {
+    if (!::GetOverlappedResult(handle, &writeCompletionOverlapped, &numberOfBytesTransferred, FALSE)) {
         numberOfBytesTransferred = 0;
         q->setError(decodeSystemError());
     }
 
-    completeAsyncWrite(numberOfBytesTransferred);
+    if (numberOfBytesTransferred > 0) {
+        writeBuffer.free(numberOfBytesTransferred);
+        emit q->bytesWritten(numberOfBytesTransferred);
+    }
+
+    writeStarted = false;
+    startAsyncWrite();
+}
+
+bool QSerialPortPrivate::startAsyncCommunication()
+{
+    Q_Q(QSerialPort);
+
+    initializeOverlappedStructure(communicationOverlapped);
+    if (!::WaitCommEvent(handle, &triggeredEventMask, &communicationOverlapped)) {
+        const QSerialPort::SerialPortError error = decodeSystemError();
+        if (error != QSerialPort::NoError) {
+            q->setError(decodeSystemError());
+            return false;
+        }
+    }
+    return true;
 }
 
 bool QSerialPortPrivate::startAsyncRead()
@@ -633,7 +666,7 @@ bool QSerialPortPrivate::startAsyncRead()
     }
 
     initializeOverlappedStructure(readCompletionOverlapped);
-    if (::ReadFile(descriptor, readChunkBuffer.data(), bytesToRead, NULL, &readCompletionOverlapped))
+    if (::ReadFile(handle, readChunkBuffer.data(), bytesToRead, NULL, &readCompletionOverlapped))
         return true;
 
     QSerialPort::SerialPortError error = decodeSystemError();
@@ -652,33 +685,63 @@ bool QSerialPortPrivate::startAsyncWrite()
 {
     Q_Q(QSerialPort);
 
-    qint64 nextSize = writeBuffer.nextDataBlockSize();
-    const char *ptr = writeBuffer.readPointer();
-
-    // no more data to write
-    if (!ptr || nextSize == 0)
+    if (writeBuffer.isEmpty() || writeStarted)
         return true;
 
-    writeSequenceStarted = true;
-
     initializeOverlappedStructure(writeCompletionOverlapped);
-    if (::WriteFile(descriptor, ptr, nextSize, NULL, &writeCompletionOverlapped))
-        return true;
+    if (!::WriteFile(handle, writeBuffer.readPointer(),
+                     writeBuffer.nextDataBlockSize(),
+                     NULL, &writeCompletionOverlapped)) {
 
-    QSerialPort::SerialPortError error = decodeSystemError();
-    if (error != QSerialPort::NoError) {
-         writeSequenceStarted = false;
+        QSerialPort::SerialPortError error = decodeSystemError();
+        if (error != QSerialPort::NoError) {
+            if (error != QSerialPort::ResourceError)
+                error = QSerialPort::WriteError;
+            q->setError(error);
+            return false;
+        }
+    }
 
-         if (error != QSerialPort::ResourceError)
-             error = QSerialPort::WriteError;
+    writeStarted = true;
+    return true;
+}
 
-         q->setError(error);
-         return false;
+bool QSerialPortPrivate::emulateErrorPolicy()
+{
+    if (!parityErrorOccurred)
+        return false;
+
+    parityErrorOccurred = false;
+
+    switch (policy) {
+    case QSerialPort::SkipPolicy:
+        readBuffer.getChar();
+        break;
+    case QSerialPort::PassZeroPolicy:
+        readBuffer.getChar();
+        readBuffer.putChar('\0');
+        emitReadyRead();
+        break;
+    case QSerialPort::IgnorePolicy:
+        return false;
+    case QSerialPort::StopReceivingPolicy:
+        emitReadyRead();
+        break;
+    default:
+        return false;
     }
 
     return true;
 }
 
+void QSerialPortPrivate::emitReadyRead()
+{
+    Q_Q(QSerialPort);
+
+    readyReadEmitted = true;
+    emit q->readyRead();
+}
+
 #endif // #ifndef Q_OS_WINCE
 
 void QSerialPortPrivate::processIoErrors(bool error)
@@ -691,7 +754,7 @@ void QSerialPortPrivate::processIoErrors(bool error)
     }
 
     DWORD errors = 0;
-    if (!::ClearCommError(descriptor, &errors, NULL)) {
+    if (!::ClearCommError(handle, &errors, NULL)) {
         q->setError(decodeSystemError());
         return;
     }
@@ -710,66 +773,11 @@ void QSerialPortPrivate::processIoErrors(bool error)
 
 #ifndef Q_OS_WINCE
 
-void QSerialPortPrivate::completeAsyncRead(DWORD numberOfBytes)
-{
-    Q_Q(QSerialPort);
-
-    if (numberOfBytes > 0) {
-
-        readBuffer.append(readChunkBuffer.left(numberOfBytes));
-
-        // Process emulate policy.
-        if ((policy != QSerialPort::IgnorePolicy) && parityErrorOccurred) {
-
-            parityErrorOccurred = false;
-
-            // Ignore received character, remove it from buffer
-            if (policy == QSerialPort::SkipPolicy) {
-                readBuffer.getChar();
-                // Force returning without emitting a readyRead() signal
-                return;
-            }
-
-            // Abort receiving
-            if (policy == QSerialPort::StopReceivingPolicy) {
-                readyReadEmitted = true;
-                emit q->readyRead();
-                return;
-            }
-
-            // Replace received character by zero
-            if (policy == QSerialPort::PassZeroPolicy) {
-                readBuffer.getChar();
-                readBuffer.putChar('\0');
-            }
-
-        }
-
-        readyReadEmitted = true;
-        emit q->readyRead();
-    }
-}
-
-void QSerialPortPrivate::completeAsyncWrite(DWORD numberOfBytes)
-{
-    Q_Q(QSerialPort);
-
-    writeBuffer.free(numberOfBytes);
-
-    if (numberOfBytes > 0)
-        emit q->bytesWritten(numberOfBytes);
-
-    if (writeBuffer.isEmpty())
-        writeSequenceStarted = false;
-    else
-        startAsyncWrite();
-}
-
 bool QSerialPortPrivate::updateDcb()
 {
     Q_Q(QSerialPort);
 
-    if (!::SetCommState(descriptor, &currentDcb)) {
+    if (!::SetCommState(handle, &currentDcb)) {
         q->setError(decodeSystemError());
         return false;
     }
@@ -780,7 +788,7 @@ bool QSerialPortPrivate::updateCommTimeouts()
 {
     Q_Q(QSerialPort);
 
-    if (!::SetCommTimeouts(descriptor, &currentCommTimeouts)) {
+    if (!::SetCommTimeouts(handle, &currentCommTimeouts)) {
         q->setError(decodeSystemError());
         return false;
     }
@@ -789,81 +797,6 @@ bool QSerialPortPrivate::updateCommTimeouts()
 
 #endif // #ifndef Q_OS_WINCE
 
-void QSerialPortPrivate::detectDefaultSettings()
-{
-    // Detect baud rate.
-    inputBaudRate = quint32(currentDcb.BaudRate);
-    outputBaudRate = inputBaudRate;
-
-    // Detect databits.
-    switch (currentDcb.ByteSize) {
-    case 5:
-        dataBits = QSerialPort::Data5;
-        break;
-    case 6:
-        dataBits = QSerialPort::Data6;
-        break;
-    case 7:
-        dataBits = QSerialPort::Data7;
-        break;
-    case 8:
-        dataBits = QSerialPort::Data8;
-        break;
-    default:
-        qWarning("%s: Unexpected data bits settings", Q_FUNC_INFO);
-        dataBits = QSerialPort::Data8;
-        break;
-    }
-
-    // Detect parity.
-    if ((currentDcb.Parity == NOPARITY) && !currentDcb.fParity)
-        parity = QSerialPort::NoParity;
-    else if ((currentDcb.Parity == SPACEPARITY) && currentDcb.fParity)
-        parity = QSerialPort::SpaceParity;
-    else if ((currentDcb.Parity == MARKPARITY) && currentDcb.fParity)
-        parity = QSerialPort::MarkParity;
-    else if ((currentDcb.Parity == EVENPARITY) && currentDcb.fParity)
-        parity = QSerialPort::EvenParity;
-    else if ((currentDcb.Parity == ODDPARITY) && currentDcb.fParity)
-        parity = QSerialPort::OddParity;
-    else {
-        qWarning("%s: Unexpected parity settings", Q_FUNC_INFO);
-        parity = QSerialPort::NoParity;
-    }
-
-    // Detect stopbits.
-    switch (currentDcb.StopBits) {
-    case ONESTOPBIT:
-        stopBits = QSerialPort::OneStop;
-        break;
-    case ONE5STOPBITS:
-        stopBits = QSerialPort::OneAndHalfStop;
-        break;
-    case TWOSTOPBITS:
-        stopBits = QSerialPort::TwoStop;
-        break;
-    default:
-        qWarning("%s: Unexpected stop bits settings", Q_FUNC_INFO);
-        stopBits = QSerialPort::OneStop;
-        break;
-    }
-
-    // Detect flow control.
-    if (!currentDcb.fOutxCtsFlow && (currentDcb.fRtsControl == RTS_CONTROL_DISABLE)
-            && !currentDcb.fInX && !currentDcb.fOutX) {
-        flow = QSerialPort::NoFlowControl;
-    } else if (!currentDcb.fOutxCtsFlow && (currentDcb.fRtsControl == RTS_CONTROL_DISABLE)
-               && currentDcb.fInX && currentDcb.fOutX) {
-        flow = QSerialPort::SoftwareControl;
-    } else if (currentDcb.fOutxCtsFlow && (currentDcb.fRtsControl == RTS_CONTROL_HANDSHAKE)
-               && !currentDcb.fInX && !currentDcb.fOutX) {
-        flow = QSerialPort::HardwareControl;
-    } else {
-        qWarning("%s: Unexpected flow control settings", Q_FUNC_INFO);
-        flow = QSerialPort::NoFlowControl;
-    }
-}
-
 QSerialPort::SerialPortError QSerialPortPrivate::decodeSystemError() const
 {
     QSerialPort::SerialPortError error;
@@ -877,6 +810,9 @@ QSerialPort::SerialPortError QSerialPortPrivate::decodeSystemError() const
     case ERROR_FILE_NOT_FOUND:
         error = QSerialPort::DeviceNotFoundError;
         break;
+    case ERROR_INVALID_NAME:
+        error = QSerialPort::DeviceNotFoundError;
+        break;
     case ERROR_ACCESS_DENIED:
         error = QSerialPort::PermissionError;
         break;
@@ -1047,7 +983,7 @@ QList<qint32> QSerialPortPrivate::standardBaudRates()
 QSerialPort::Handle QSerialPort::handle() const
 {
     Q_D(const QSerialPort);
-    return d->descriptor;
+    return d->handle;
 }
 
 QT_END_NAMESPACE
